/* include/validator/validator-config.h.  Generated from validator-config.h.in by configure.  */
/* include/validator/validator-config.h.in.  Generated from configure.in by autoheader.  */

/* Define if gethostbyaddr expects a char, int, int */
/* #undef GETHOSTBYADDR_USES_CHAR_INT */

/* Define if gethostbyaddr expects void, int, int */
/* #undef GETHOSTBYADDR_USES_VOID_INT */

/* Define if gethostbyaddr expects void, socklen_t, int */
#define GETHOSTBYADDR_USES_VOID_SOCKLEN 1

/* Define if getnameinfo expects a size_t and an int */
/* #undef GETNAMEINFO_USES_SIZET_AND_INT */

/* Define if getnameinfo expects a socklen_t and an int */
#define GETNAMEINFO_USES_SOCKLEN_AND_INT 1

/* Define if getnameinfo expects a socklen_t and a unsigned int */
/* #undef GETNAMEINFO_USES_SOCKLEN_AND_UINT */

/* Define to 1 if you have the <arpa/inet.h> header file. */
#define HAVE_ARPA_INET_H 1

/* Define to 1 if you have the <arpa/nameser_compat.h> header file. */
#define HAVE_ARPA_NAMESER_COMPAT_H 1

/* Define to 1 if you have the <arpa/nameser.h> header file. */
#define HAVE_ARPA_NAMESER_H 1

/* Define to 1 if you have the <crypto/sha2.h> header file. */
/* #undef HAVE_CRYPTO_SHA2_H */

/* Define to 1 if you have the <ctype.h> header file. */
#define HAVE_CTYPE_H 1

/* Define to 1 if you have the declaration of `getopt_long', and to 0 if you
   don't. */
#define HAVE_DECL_GETOPT_LONG 1

/* Define to 1 if you have the declaration of `getopt_long_only', and to 0 if
   you don't. */
#define HAVE_DECL_GETOPT_LONG_ONLY 1

/* Define to 1 if you have the declaration of `ns_name_ntop', and to 0 if you
   don't. */
#define HAVE_DECL_NS_NAME_NTOP 1

/* Define to 1 if you have the declaration of `ns_name_pton', and to 0 if you
   don't. */
#define HAVE_DECL_NS_NAME_PTON 1

/* Define to 1 if you have the declaration of `ns_name_unpack', and to 0 if
   you don't. */
#define HAVE_DECL_NS_NAME_UNPACK 1

/* Define to 1 if you have the declaration of `ns_parse_ttl', and to 0 if you
   don't. */
#define HAVE_DECL_NS_PARSE_TTL 1

/* Define to 1 if you have the declaration of `ns_r_badsig', and to 0 if you
   don't. */
#define HAVE_DECL_NS_R_BADSIG 1

/* Define to 1 if you have the declaration of `ns_samename', and to 0 if you
   don't. */
#define HAVE_DECL_NS_SAMENAME 1

/* Define to 1 if you have the declaration of `ns_t_a', and to 0 if you don't.
   */
#define HAVE_DECL_NS_T_A 1

/* Define to 1 if you have the declaration of `ns_t_aaaa', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_AAAA 1

/* Define to 1 if you have the declaration of `ns_t_dlv', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_DLV 1

/* Define to 1 if you have the declaration of `ns_t_dnskey', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_DNSKEY 1

/* Define to 1 if you have the declaration of `ns_t_ds', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_DS 1

/* Define to 1 if you have the declaration of `ns_t_kx', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_KX 1

/* Define to 1 if you have the declaration of `ns_t_nsec', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_NSEC 1

/* Define to 1 if you have the declaration of `ns_t_nsec3', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_NSEC3 1

/* Define to 1 if you have the declaration of `ns_t_rrsig', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_RRSIG 1

/* Define to 1 if you have the declaration of `ns_t_tlsa', and to 0 if you
   don't. */
#define HAVE_DECL_NS_T_TLSA 1

/* Define to 1 if you have the declaration of `p_rcode', and to 0 if you
   don't. */
#define HAVE_DECL_P_RCODE 1

/* Define to 1 if you have the declaration of `p_section', and to 0 if you
   don't. */
#define HAVE_DECL_P_SECTION 0

/* Define to 1 if you have the declaration of `struct addrinfo', and to 0 if
   you don't. */
#define HAVE_DECL_STRUCT_ADDRINFO 0

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define if libcrypto implements the ECDSA algorithm. */
#define HAVE_ECDSA 1

/* Define to 1 if you have the <endian.h> header file. */
#define HAVE_ENDIAN_H 1

/* Define to 1 if you have the <errno.h> header file. */
#define HAVE_ERRNO_H 1

/* Define to 1 if you have the `flock' function. */
#define HAVE_FLOCK 1

/* Define to 1 if you have the `freeaddrinfo' function. */
#define HAVE_FREEADDRINFO 1

/* Define to 1 if you have the `gethostbyname2' function. */
#define HAVE_GETHOSTBYNAME2 1

/* Define to 1 if you have the `getifaddrs' function. */
#define HAVE_GETIFADDRS 1

/* Define to 1 if you have the <getopt.h> header file. */
#define HAVE_GETOPT_H 1

/* Define to 1 if you have the `gettimeofday' function. */
#define HAVE_GETTIMEOFDAY 1

/* Define to 1 if you have the `gmtime_r' function. */
#define HAVE_GMTIME_R 1

/* Define to 1 if you have the `hstrerror' function. */
#define HAVE_HSTRERROR 1

/* Define to 1 if you have the <ifaddrs.h> header file. */
#define HAVE_IFADDRS_H 1

/* Define to 1 if you have the `inet_nsap_ntoa' function. */
#define HAVE_INET_NSAP_NTOA 1

/* Define to 1 if the system has the type `int16_t'. */
#define HAVE_INT16_T 1

/* Define to 1 if the system has the type `int32_t'. */
#define HAVE_INT32_T 1

/* Define to 1 if the system has the type `int64_t'. */
#define HAVE_INT64_T 1

/* Define to 1 if the system has the type `int8_t'. */
#define HAVE_INT8_T 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <libgen.h> header file. */
#define HAVE_LIBGEN_H 1

/* Define to 1 if you have the `nsl' library (-lnsl). */
#define HAVE_LIBNSL 1

/* Define to 1 if you have the `socket' library (-lsocket). */
/* #undef HAVE_LIBSOCKET */

/* Define to 1 if you have the <limits.h> header file. */
#define HAVE_LIMITS_H 1

/* Define to 1 if you have the `localtime_r' function. */
#define HAVE_LOCALTIME_R 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <netdb.h> header file. */
#define HAVE_NETDB_H 1

/* Define to 1 if you have the <netinet/in.h> header file. */
#define HAVE_NETINET_IN_H 1

/* Define to 1 if you have the <net/if.h> header file. */
#define HAVE_NET_IF_H 1

/* Define to 1 if the system has the type `ns_cert_types'. */
#define HAVE_NS_CERT_TYPES 1

/* Define to 1 if you have the <openssl/ecdsa.h> header file. */
#define HAVE_OPENSSL_ECDSA_H 1

/* Define to 1 if you have the `pselect' function. */
#define HAVE_PSELECT 1

/* Define to 1 if you have the <pthread.h> header file. */
#define HAVE_PTHREAD_H 1

/* Define to 1 if you have the <resolv.h> header file. */
#define HAVE_RESOLV_H 1

/* Define if libcrypto implements the SHA-2 suite of algorithms. */
#define HAVE_SHA_2 1

/* Define to 1 if the system has the type `size_t'. */
#define HAVE_SIZE_T 1

/* Define to 1 if you have the `snprintf' function. */
#define HAVE_SNPRINTF 1

/* Define to 1 if the system has the type `ssize_t'. */
#define HAVE_SSIZE_T 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the `strerror_r' function. */
#define HAVE_STRERROR_R 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the `strncasecmp' function. */
#define HAVE_STRNCASECMP 1

/* Define to 1 if you have the `strtok_r' function. */
#define HAVE_STRTOK_R 1

/* Define to 1 if you have the `getrlimit' function. */
#define HAVE_GETRLIMIT 1

/* Define to 1 if you have the `setrlimit' function. */
#define HAVE_SETRLIMIT 1

/* Define to 1 if you have the <syslog.h> header file. */
#define HAVE_SYSLOG_H 1

/* Define to 1 if you have the <sys/fcntl.h> header file. */
#define HAVE_SYS_FCNTL_H 1

/* Define to 1 if you have the <sys/file.h> header file. */
#define HAVE_SYS_FILE_H 1

/* Define to 1 if you have the <sys/filio.h> header file. */
/* #undef HAVE_SYS_FILIO_H */

/* Define to 1 if you have the <sys/ioctl.h> header file. */
#define HAVE_SYS_IOCTL_H 1

/* Define to 1 if you have the <sys/param.h> header file. */
#define HAVE_SYS_PARAM_H 1

/* Define to 1 if you have the <sys/select.h> header file. */
#define HAVE_SYS_SELECT_H 1

/* Define to 1 if you have the <sys/socket.h> header file. */
#define HAVE_SYS_SOCKET_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/time.h> header file. */
#define HAVE_SYS_TIME_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <sys/resource.h> header file. */
#define HAVE_SYS_RESOURCE_H 1

/* Define to 1 if the system has the type `uint16_t'. */
#define HAVE_UINT16_T 1

/* Define to 1 if the system has the type `uint32_t'. */
#define HAVE_UINT32_T 1

/* Define to 1 if the system has the type `uint64_t'. */
#define HAVE_UINT64_T 1

/* Define to 1 if the system has the type `uint8_t'. */
#define HAVE_UINT8_T 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if the system has the type `u_char'. */
#define HAVE_U_CHAR 1

/* Define to 1 if the system has the type `u_int16_t'. */
#define HAVE_U_INT16_T 1

/* Define to 1 if the system has the type `u_int32_t'. */
#define HAVE_U_INT32_T 1

/* Define to 1 if the system has the type `u_int64_t'. */
#define HAVE_U_INT64_T 1

/* Define to 1 if the system has the type `u_int8_t'. */
#define HAVE_U_INT8_T 1

/* Define to 1 if the system has the type `u_long'. */
#define HAVE_U_LONG 1

/* Define to 1 if the system has the type `u_short'. */
#define HAVE_U_SHORT 1

/* Configure options */
#define LIBVAL_CONFIGURE_OPTIONS " '--disable-option-checking' '--prefix=/usr' '--exec_prefix=/usr' '--mandir=/usr/share/man' '--sysconfdir=/etc' '--with-ipv6' '--with-nsec3' '--with-dlv' '--with-perl-build-args=INSTALLDIRS=vendor' 'CFLAGS=-march=i486 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=2 -Wformat -Werror=format-security         -fstack-clash-protection ' 'LDFLAGS=-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now' '--cache-file=/dev/null' '--srcdir=.'"

/* Define if DLV is required. */
#define LIBVAL_DLV 1

/* Define if inline configuration files are to be used. */
#define LIBVAL_INLINE_POLICY 1

/* Define if NSEC3 is required. */
#define LIBVAL_NSEC3 1

/* Define to the sub-directory in which libtool stores uninstalled libraries.
   */
#define LT_OBJDIR ".libs/"

/* Define if <arpa/nameser.h> has HEADER. */
#define NAMESER_HAS_HEADER 1

/* Define validator hosts file */
#define ETC_HOSTS "/etc/hosts"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "dnssec-tools-users@dnssec-tools.org"

/* Define to the full name of this package. */
#define PACKAGE_NAME "dnssec-tools-validator"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "dnssec-tools-validator 0.3"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "dnssec-tools-validator"

/* Define to the home page for this package. */
#define PACKAGE_URL ""

/* Define to the version of this package. */
#define PACKAGE_VERSION "0.3"

/* Define if gethostbyname_r returns a struct hostent */
/* #undef R_FUNCS_RETURN_STRUCT */

/* The size of `int', as computed by sizeof. */
#define SIZEOF_INT 4

/* The size of `long', as computed by sizeof. */
#define SIZEOF_LONG 4

/* The size of `short', as computed by sizeof. */
#define SIZEOF_SHORT 2

/* *s*printf() functions are char* */
/* #undef SPRINTF_CHAR */

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Define if struct __ns_msg had _msg_ptr. */
#define STRUCT___NS_MSG_HAS__MSG_PTR 1

/* Define if struct __ns_msg had _ptr. */
/* #undef STRUCT___NS_MSG_HAS__PTR */

/* validator testcases configuration file */
#define VALIDATOR_TESTCASES "/etc/dnssec-tools/validator-testcases"

/* dnsval configuration file */
#define VAL_CONFIGURATION_FILE "/etc/dnssec-tools/dnsval.conf"

/* Define if IPV6 is required. */
#define VAL_IPV6 1

/* Define if you do not want to use threads. */
/* #undef VAL_NO_THREADS */

/* Path to resolver configuration */
#define VAL_RESOLV_CONF "/etc/dnssec-tools/resolv.conf"

/* Path to root hints config file */
#define VAL_ROOT_HINTS "/etc/dnssec-tools/root.hints"


#include "validator/validator-compat.h"

